@@ -1,7 +1,7 @@
 import json
 import re
 from urllib.parse import parse_qsl
-from typing import TYPE_CHECKING, Type, Tuple, TypeVar, Union, Dict
+from typing import TYPE_CHECKING, TypeVar, Union
 
 from lagrange.client.message.decoder import parse_grp_msg, parse_friend_msg
 from lagrange.pb.message.msg_push import MsgPush
@@ -14,13 +14,22 @@ from lagrange.pb.status.group import (
     MemberJoinRequest,
     MemberRecallMsg,
     GroupSub20Head,
+    PBBotGrayTip,
     PBGroupAlbumUpdate,
+    PBGroupBotAdded,
+    PBGroupInvite,
+    PBSelfJoinInGroup,
 )
+from lagrange.pb.status.friend import PBFriendRecall, PBFriendRequest
 from lagrange.utils.binary.protobuf import proto_decode, ProtoStruct, proto_encode
 from lagrange.utils.binary.reader import Reader
 from lagrange.utils.operator import unpack_dict, timestamp
 
 from ..events.group import (
+    BotGrayTip,
+    GroupBotAdded,
+    GroupBotJoined,
+    GroupInvite,
     GroupMemberGotSpecialTitle,
     GroupMemberJoined,
     GroupMemberJoinRequest,
@@ -30,9 +39,13 @@ from ..events.group import (
     GroupRecall,
     GroupNudge,
     GroupReaction,
+    GroupSelfJoined,
+    GroupSelfRequireReject,
     GroupSign,
     GroupAlbumUpdate,
+    GroupMemberJoinedByInvite,
 )
+from ..events.friend import FriendRecall, FriendRequest
 from ..wtlogin.sso import SSOPacket
 from .log import logger
 
@@ -42,30 +55,41 @@ if TYPE_CHECKING:
 T = TypeVar("T", bound=ProtoStruct)
 
 
-def unpack(buf2: bytes, decoder: Type[T]) -> Tuple[int, T]:
+def unpack(buf2: bytes, decoder: type[T]) -> tuple[int, T]:
     reader = Reader(buf2)
     grp_id = reader.read_u32()
     reader.read_u8()
     return grp_id, decoder.decode(reader.read_bytes_with_length("u16", False))
 
 
+def parse_quoted_str(string: str) -> list[dict[str, Union[str, int]]]:
+    """
+    example data: 'test data <{"cmd": 1, "text": "updated"}> <{"cmd": 2}>!'
+    to: [{"cmd": 1, "text": "updated"}, {"cmd": 2}]
+    """
+    return [
+        json.loads(el)
+        for el in re.findall(r"<(\{.*?})>", string)
+    ]
+
+
 async def msg_push_handler(client: "Client", sso: SSOPacket):
     pkg = MsgPush.decode(sso.data).body
     typ = pkg.content_head.type
     sub_typ = pkg.content_head.sub_type
 
-    logger.debug("msg_push received, type: {}.{}".format(typ, sub_typ))
+    logger.debug(f"msg_push received, type: {typ}.{sub_typ}")
     if typ == 82:  # grp msg
         return await parse_grp_msg(client, pkg)
     elif typ in [166, 208, 529]:  # frd msg
-        return await parse_friend_msg(client, pkg)
+        if pkg.message:
+            return await parse_friend_msg(client, pkg)
     elif typ == 33:  # member joined
         pb = MemberChanged.decode(pkg.message.buf2)
         return GroupMemberJoined(
-            grp_id=pkg.response_head.from_uin,
-            uin=pb.uin,
-            uid=pb.uid,
-            join_type=pb.join_type,
+            grp_id=pb.uin,
+            uid=pb.uid,  # right u cant get uin
+            join_type=pb.join_type_new,
         )
     elif typ == 34:  # member exit
         pb = MemberChanged.decode(pkg.message.buf2)
@@ -78,24 +102,60 @@ async def msg_push_handler(client: "Client", sso: SSOPacket):
         )
     elif typ == 84:
         pb = MemberJoinRequest.decode(pkg.message.buf2)
-        return GroupMemberJoinRequest(
-            grp_id=pb.grp_id, uid=pb.uid, answer=pb.request_field
-        )
+        return GroupMemberJoinRequest(grp_id=pb.grp_id, uid=pb.uid, answer=pb.request_field)
+    elif typ == 85:
+        pb = PBSelfJoinInGroup.decode(pkg.message.buf2)
+        return GroupSelfJoined(grp_id=pb.gid, op_uid=pb.operator_uid)
+    elif typ == 86:
+        reader = Reader(pkg.message.buf2)
+        grp_id = reader.read_u32()  # it should have more infomation,but i cant guess it
+        return GroupSelfRequireReject(grp_id, "")
+    elif typ == 87:
+        pb = PBGroupInvite.decode(pkg.message.buf2)
+        return GroupInvite(grp_id=pb.gid, invitor_uid=pb.invitor_uid)
+    elif typ == 167:
+        print(pkg.message.encode().hex())
     elif typ == 525:
         pb = MemberInviteRequest.decode(pkg.message.buf2)
         if pb.cmd == 87:
             inn = pb.info.inner
-            return GroupMemberJoinRequest(
-                grp_id=inn.grp_id, uid=inn.uid, invitor_uid=inn.invitor_uid
+            return GroupMemberJoinRequest(grp_id=inn.grp_id, uid=inn.uid, invitor_uid=inn.invitor_uid)
+    elif typ == 0x210:  # friend event, 528 / group file upload notice event
+        if sub_typ == 35:  # friend request
+            pb = PBFriendRequest.decode(pkg.message.buf2)
+            return FriendRequest(
+                pkg.response_head.from_uin,
+                pb.info.from_uid,
+                pkg.response_head.to_uin,
+                pb.info.to_uid,
+                pb.info.verify,
+                pb.info.source,
             )
-    elif typ == 0x210:  # friend event / group file upload notice event
-        logger.debug("unhandled friend event / group file upload notice event: %s" % pkg)  # TODO: paste
+        elif sub_typ == 138:  # friend recall
+            pb = PBFriendRecall.decode(pkg.message.buf2)
+            return FriendRecall(
+                pkg.response_head.from_uin,
+                pb.info.from_uid,
+                pkg.response_head.to_uin,
+                pb.info.to_uid,
+                pb.info.seq,
+                pb.info.random,
+                pb.info.time,
+            )
+        if sub_typ == 368:
+            pass
+            # print(pkg.message.encode().hex())
+        logger.debug(f"unhandled friend event / group file upload notice event: {pkg}")  # TODO: paste
     elif typ == 0x2DC:  # grp event, 732
+        if sub_typ == 1:
+            # print(pkg.encode().hex())
+            pass
         if sub_typ == 20:  # nudge and group_sign(群打卡)
             if pkg.message:
                 grp_id, pb = unpack(pkg.message.buf2, GroupSub20Head)
-                attrs: Dict[str, Union[str, int]] = {}
-                for x in pb.body.attrs:  # type: dict[bytes, bytes]
+                attrs: dict[str, Union[str, int]] = {}
+                for x in pb.body.attrs:
+                    x: dict[bytes, bytes]
                     k, v = x.values()
                     if isinstance(v, dict):
                         v = proto_encode(v)
@@ -103,14 +163,30 @@ async def msg_push_handler(client: "Client", sso: SSOPacket):
                         attrs[k.decode()] = int(v.decode())
                     else:
                         attrs[k.decode()] = v.decode()
-                if pb.body.type == 12:
+                if pb.body.f2 == 19217:
+                    return GroupBotJoined(
+                        grp_id,
+                        attrs["mqq_uin"],
+                        attrs["nick_name"],
+                        attrs["robot_name"],
+                        attrs["robot_schema"],
+                        attrs["user_schema"],
+                    )
+                if pb.body.type == 1:
+                    if "invitor" in attrs:
+                        # reserve: attrs["msg_nums"]
+                        return GroupMemberJoinedByInvite(grp_id, attrs["invitor"], attrs["invitee"])
+                    elif "user" in attrs and "uin" in attrs:
+                        # todo: 群代办
+                        pass
+                    else:
+                        raise TypeError(f"Unhandled GrayTips with attribute: {attrs}")
+                elif pb.body.type == 12:
                     return GroupNudge(
                         grp_id,
                         attrs["uin_str1"],
                         attrs["uin_str2"],
-                        attrs["action_str"]
-                        if "action_str" in attrs
-                        else attrs["alt_str1"],  # ?
+                        attrs["action_str"] if "action_str" in attrs else attrs["alt_str1"],  # ?
                         attrs["suffix_str"],
                         attrs,
                         pb.body.attrs_xml,
@@ -125,19 +201,19 @@ async def msg_push_handler(client: "Client", sso: SSOPacket):
                         pb.body.attrs_xml,
                     )
                 else:
-                    raise ValueError(
-                        f"unknown type({pb.body.type}) on GroupSub20: {attrs}"
-                    )
+                    raise ValueError(f"unknown type({pb.body.type}) f2({pb.body.f2}) on GroupSub20: {attrs}")
             else:
                 # print(pkg.encode().hex(), 2)
                 return
         elif sub_typ == 16:  # rename & special_title & reaction
             if pkg.message:
                 grp_id, pb = unpack(pkg.message.buf2, GroupSub16Head)
+                if pb.flag is None:
+                    _, pb = unpack(pkg.message.buf2, PBBotGrayTip)  # 傻逼tx，我13号位呢
+                    return BotGrayTip(grp_id, pb.body.message)
                 if pb.flag == 6:  # special_title
                     body = MemberGotTitleBody.decode(pb.body)
-                    for el in re.findall(r"<(\{.*?})>", body.string):
-                        el = json.loads(el)
+                    for el in parse_quoted_str(body.string):
                         if el["cmd"] == 1:
                             title = el["text"]
                             url = el["data"]
@@ -172,20 +248,22 @@ async def msg_push_handler(client: "Client", sso: SSOPacket):
                     )
                 elif pb.flag == 23:  # 群幸运字符？
                     pass
+                elif pb.flag == 21:  # 位置实时分享（不完整）
+                    pass
+                    #print(sso.data.hex())
                 elif pb.flag == 37:  # 群相册上传（手Q专用:(）
-                    _, pb = unpack(
-                        pkg.message.buf2, PBGroupAlbumUpdate
-                    )  # 塞 就硬塞，可以把你的顾辉盒也给塞进来
+                    _, pb = unpack(pkg.message.buf2, PBGroupAlbumUpdate)  # 塞 就硬塞，可以把你的顾辉盒也给塞进来
                     q = dict(parse_qsl(pb.body.args))
                     return GroupAlbumUpdate(
                         grp_id=pb.grp_id,
                         timestamp=pb.timestamp,
                         image_id=q["i"],
                     )
+                elif pb.flag == 38:
+                    _, pb = unpack(pkg.message.buf2, PBGroupBotAdded)
+                    return GroupBotAdded(pb.body.grp_id, pb.body.bot_uid_1 or pb.body.bot_uid_2)
                 else:
-                    raise ValueError(
-                        f"Unknown subtype_12 flag: {pb.flag}: {pb.body.hex() if pb.body else pb}"
-                    )
+                    raise ValueError(f"Unknown subtype_12 flag: {pb.flag}: {pb.body.hex() if pb.body else pb}")
         elif sub_typ == 17:  # recall
             grp_id, pb = unpack(pkg.message.buf2, MemberRecallMsg)
 
@@ -201,10 +279,10 @@ async def msg_push_handler(client: "Client", sso: SSOPacket):
         elif sub_typ == 12:  # mute
             info = proto_decode(pkg.message.buf2)
             return GroupMuteMember(
-                grp_id=info[1],
-                operator_uid=info[4].decode(),
-                target_uid=unpack_dict(info, "5.3.1", b"").decode(),
-                duration=unpack_dict(info, "5.3.2"),
+                grp_id=info.into(1, int),
+                operator_uid=info.into(4, bytes).decode(),
+                target_uid=unpack_dict(info.proto, "5.3.1", b"").decode(),
+                duration=unpack_dict(info.proto, "5.3.2"),
             )
         elif sub_typ == 21:  # set/unset essence msg
             pass  # todo
@@ -213,9 +291,7 @@ async def msg_push_handler(client: "Client", sso: SSOPacket):
                 "unknown sub_type %d: %s"
                 % (
                     sub_typ,
-                    pkg.message.buf2.hex()
-                    if getattr(pkg.message, "buf2", None)
-                    else pkg,
+                    pkg.message.buf2.hex() if getattr(pkg.message, "buf2", None) else pkg,
                 )
             )
     else:
